package com.jkcredit.stat.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.extra.mail.MailAccount;
import cn.hutool.extra.mail.MailUtil;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
public class UsedCarService {
    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @Value("${mail.send.from}")
    private String sendFrom;
    @Value("${mail.send.to}")
    private String sendTo;
    @Value("${mail.host}")
    private String host;
    @Value("${mail.username}")
    private String username;
    @Value("${mail.authentication}")
    private String authentication;
    @Value("${bill.path}")
    private String billPath;

    public static final String REDIS_KEY_USED_CAR_INSURANCE = "risk_sec_engine_ids_USED_CAR_INSURANCE";
    public static final String REDIS_KEY_USED_CAR_VEHICLE = "risk_sec_engine_ids_USED_CAR_VEHICLE";
    public static final String REDIS_KEY_USED_CAR_INSURANCE_VEHICLE = "risk_sec_engine_ids_USED_CAR_INSURANCE_VEHICLE";
    private static final String FOLDER_NAME_INSURANCE = "insurance";
    private static final String FOLDER_NAME_VEHICLE_INFO = "vehicleInfo";
    private static final String FOLDER_NAME_INSURANCE_VEHICLE_INFO = "insuranceVehicleInfo";
    private static final String CHAR_SET = "utf-8";

    public void sendBill() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMM");
        String fileName = simpleDateFormat.format(new Date()) + ".txt";
        // 判断是否是每月1号
        if (DateUtil.dayOfMonth(new Date()) == 1) {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.MONTH, calendar.get(Calendar.MONTH) - 1);
            fileName = simpleDateFormat.format(calendar.getTime()) + ".txt";
        }
        String insuranceBillPath = getFilePath(FOLDER_NAME_INSURANCE, fileName);
        String vehicleBillPath = getFilePath(FOLDER_NAME_VEHICLE_INFO, fileName);
        String insuranceVehicleBillPath = getFilePath(FOLDER_NAME_INSURANCE_VEHICLE_INFO, fileName);
        List<String> insuranceMembers = FileUtil.readLines(insuranceBillPath, CHAR_SET);
        List<String> insuranceVehicleMembers = FileUtil.readLines(insuranceVehicleBillPath, CHAR_SET);
        List<String> vehicleMembers = FileUtil.readLines(vehicleBillPath, CHAR_SET);

        Map<String, Integer> insuranceBill = getBillDetail(insuranceMembers);
        Map<String, Integer> vehicleBill = getBillDetail(vehicleMembers);
        Map<String, Integer> insuranceVehicleBill = getBillDetail(insuranceVehicleMembers);

        log.info("insuranceBill:{}", insuranceBill);
        log.info("vehicleBill:{}", vehicleBill);
        log.info("insuranceVehicleBill:{}", insuranceVehicleBill);

        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("银保信二手车账单：").append(System.getProperty("line.separator"));
        stringBuilder.append("保险信息：").append(System.getProperty("line.separator"));
        formatBillMessage(stringBuilder, insuranceBill);

        stringBuilder.append("车辆信息：").append(System.getProperty("line.separator"));
        formatBillMessage(stringBuilder, vehicleBill);

        stringBuilder.append("车辆保险信息：").append(System.getProperty("line.separator"));
        formatBillMessage(stringBuilder, insuranceVehicleBill);

        sendMail(stringBuilder.toString(), "银保信二手车账单");
    }

    public void saveBill() {
        saveChargeBill(REDIS_KEY_USED_CAR_INSURANCE, FOLDER_NAME_INSURANCE);
        saveChargeBill(REDIS_KEY_USED_CAR_VEHICLE, FOLDER_NAME_VEHICLE_INFO);
        saveChargeBill(REDIS_KEY_USED_CAR_INSURANCE_VEHICLE, FOLDER_NAME_INSURANCE_VEHICLE_INFO);
    }

    private Map<String, Integer> getBillDetail(List<String> members) {
        Map<String, Integer> billMap = new HashMap<>();
        for (String str : members) {
            Map<String, Object> detailMap = JSONUtil.toBean(str, Map.class);
            for (Map.Entry<String, Object> entry : detailMap.entrySet()) {
                if (entry.getKey().equals("requestId")) {
                    continue;
                }
                int count = billMap.get(entry.getKey()) == null ? 0 : billMap.get(entry.getKey());
                int detailCount = (Integer) entry.getValue();
                billMap.put(entry.getKey(), count + detailCount);
            }
        }
        return billMap;
    }

    private void formatBillMessage(StringBuilder builder, Map<String, Integer> billMap) {
        for (Map.Entry<String, Integer> entry : billMap.entrySet()) {
            builder
                    .append(entry.getKey())
                    .append(":")
                    .append(entry.getValue())
                    .append(System.getProperty("line.separator"));
        }
    }

    public static void main(String[] args) {
        MailAccount mailAccount = new MailAccount();
        mailAccount.setAuth(true);
        mailAccount.setFrom("jkcredit2021@163.com");
        mailAccount.setHost("smtp.163.com");
        mailAccount.setUser("jkcredit2021@163.com");
        mailAccount.setPass("MAMBDVCOVNHQDUSL");
        MailUtil.send(mailAccount, "xsl@jkcredit.com", "subject", "content", false);
    }

    private void sendMail(String content, String subject) {
        MailAccount mailAccount = new MailAccount();
        mailAccount.setAuth(true);
        mailAccount.setFrom(sendFrom);
        mailAccount.setHost(host);
        mailAccount.setUser(username);
        mailAccount.setPass(authentication);
        MailUtil.send(mailAccount, sendTo, subject, content, false);
    }

    private void saveChargeBill(String redisKey, String folderName) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMM");
        String fileName = simpleDateFormat.format(new Date()) + ".txt";

        Set<String> insuranceMembers = stringRedisTemplate.boundSetOps(redisKey).members();
        log.info("folder:{}, saveBill:{}", folderName, insuranceMembers);
        String billPath = getFilePath(folderName, fileName);
        List<String> insuranceList = new ArrayList<>(insuranceMembers);
        try {
            FileUtil.appendUtf8Lines(insuranceList, billPath);
            for (String s : insuranceMembers) {
                stringRedisTemplate.boundSetOps(redisKey).remove(s);
            }
        } catch (Exception ioe) {
            log.info("writeBillException:" + ioe);
        }
    }

    private String getFilePath(String folderName, String fileName) {
        StringBuilder insuranceBillPath = new StringBuilder(billPath)
                .append(File.separator)
                .append(folderName)
                .append(File.separator)
                .append(fileName);
        return insuranceBillPath.toString();
    }
}
